"use strict";

var debug = require("debug")("access-limit-http-proxy");
var Thin = require('thin');
var assert = require('assert');
var proxy = new Thin();
var countMap = new Map();
/**
 * @typedef {Object} TargetInterface
 * @type {{predicate: ((count))}}
 */
var TargetInterface = {
    predicate: function predicate(count) {
        return true;
    }
};

module.exports = function (targets) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var port = options.port || 8989;
    var intercept = function intercept(req, res, next) {
        var requestURL = req.url.slice(1);
        if (!targets[requestURL]) {
            return void next();
        }
        var predicate = targets[requestURL].predicate;
        assert(typeof predicate === "function", "target should have predicate");
        var currentCount = countMap.get(requestURL) || 0;
        var count = currentCount + 1;
        debug("requestURL:", requestURL, "count:", count);
        countMap.set(requestURL, count);
        if (predicate(count)) {
            debug("redirect", requestURL);
            res.writeHead(302, {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Request-Method': '*',
                'Access-Control-Allow-Methods': 'OPTIONS, GET',
                'Access-Control-Allow-Headers': '*',
                Location: requestURL
            });
            res.end();
        }
        res.writeHead(500, { 'Content-Type': 'text/plain' });
        res.end('500 Internal Server Error(Fake)');
    };
    proxy.use(intercept);
    proxy.listen(port, 'localhost', function (err) {
        if (err) {
            console.error(err);
        }
    });
    console.log("Listen http://localhost:" + port);
    console.log("Use as Proxy like this: http://localhost:" + port + "/http://example.com");
};
//# sourceMappingURL=access-limit-http-proxy.js.map